"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Errors_1 = require("../Errors/Errors");
const Metadata_1 = require("../Metadata/Metadata");
/**
 * default options
 */
const defaultOptions = {
    cascadeInsert: false,
    cascadeUpdate: false
};
/**
 * Defines single or multiple relation property
 *
 * @export
 * @param type
 * @param [options]
 * @returns
 */
function RelationProperty(type, options) {
    return function (target, propertyKey) {
        const designType = Reflect.getMetadata("design:type", target, propertyKey);
        const redisPropertyName = options && options.propertyName ? options.propertyName : propertyKey;
        const relationTypes = type();
        let propertyType;
        if (Array.isArray(relationTypes)) {
            [, propertyType] = relationTypes;
        }
        else {
            if (designType === Object) {
                throw new Errors_1.MetadataError(target.constructor, `Relation's ${propertyKey} property type is detected as simple Object. This is error. Specify property type explicitly`);
            }
            propertyType = designType;
        }
        const finalOptions = Object.assign({}, defaultOptions, options);
        const properties = Reflect.getMetadata(Metadata_1.REDIS_PROPERTIES, target.constructor) || [];
        properties.push({
            isRelation: true,
            propertyName: propertyKey,
            propertyRedisName: redisPropertyName,
            propertyType: propertyType,
            relationTypeFunc: type,
            relationOptions: finalOptions
        });
        Reflect.defineMetadata(Metadata_1.REDIS_PROPERTIES, properties, target.constructor);
    };
}
exports.RelationProperty = RelationProperty;
//# sourceMappingURL=RelationProperty.js.map