import { Connection } from "../Connection/Connection";
export declare class RedisTestMonitor {
    private _calls;
    /**
     * Redis client
     *
     * @private
     */
    private client;
    /**
     * Filter calls for specific redis db (each test connection connects to different db)
     *
     * @private
     */
    private filterDbNumber?;
    /**
     * All monitor calls in format [request, response]
     *
     * @readonly
     */
    readonly calls: string[][][];
    /**
     * Request only calls
     *
     * @readonly
     */
    readonly requests: string[][];
    /**
     * Responses only
     *
     * @readonly
     */
    readonly responses: string[][];
    /**
     * Release monitor
     *
     * @returns
     */
    release(): Promise<void>;
    /**
     * Reset monitor calls
     * @param timeToWaitMs Wait some time before clearing
     *
     */
    clearMonitorCalls(timeToWaitMs?: number): Promise<void>;
    /**
     * Wait some time
     *
     * @param timeToWaitMs
     * @returns
     */
    wait(timeToWaitMs: number): Promise<void>;
    /**
     * Logger
     *
     * @protected
     * @param time
     * @param commandArgs
     * @param reply
     * @returns
     */
    protected monitor(time: number, commandArgs: string[], reply: string): void;
    /**
     * Create monitor
     *
     * @static
     * @param [connection] Monitor only for requests/replies for given connection db
     * @returns
     */
    static create(connection?: Connection): Promise<RedisTestMonitor>;
}
