"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const redis_1 = require("./redis");
class RedisTestMonitor {
    constructor() {
        // Array of calls -> results without any timestamps
        this._calls = [];
    }
    /**
     * All monitor calls in format [request, response]
     *
     * @readonly
     */
    get calls() {
        return this._calls;
    }
    /**
     * Request only calls
     *
     * @readonly
     */
    get requests() {
        return this._calls.map(([request, result]) => request);
    }
    /**
     * Responses only
     *
     * @readonly
     */
    get responses() {
        return this._calls.map(([request, result]) => result);
    }
    /**
     * Release monitor
     *
     * @returns
     */
    release() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.quitAsync();
        });
    }
    /**
     * Reset monitor calls
     * @param timeToWaitMs Wait some time before clearing
     *
     */
    clearMonitorCalls(timeToWaitMs) {
        return __awaiter(this, void 0, void 0, function* () {
            if (timeToWaitMs) {
                yield this.wait(timeToWaitMs);
            }
            this._calls = [];
        });
    }
    /**
     * Wait some time
     *
     * @param timeToWaitMs
     * @returns
     */
    wait(timeToWaitMs) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise(resolve => setTimeout(resolve, timeToWaitMs));
        });
    }
    /**
     * Logger
     *
     * @protected
     * @param time
     * @param commandArgs
     * @param reply
     * @returns
     */
    monitor(time, commandArgs, reply) {
        const repliesArgs = reply.split(" "); // 1500637727.256744 [12 172.20.0.1:48986] \"multi\"
        const justReply = repliesArgs.splice(3).map(r => r.slice(1, -1));
        const dbNumber = repliesArgs[1].slice(1); // [12 172.20.0.1:48986]
        if (this.filterDbNumber) {
            if (!dbNumber) {
                return;
            }
            if (dbNumber !== this.filterDbNumber) {
                return;
            }
        }
        this._calls.push([commandArgs, justReply]);
    }
    /**
     * Create monitor
     *
     * @static
     * @param [connection] Monitor only for requests/replies for given connection db
     * @returns
     */
    static create(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const instance = new RedisTestMonitor();
            if (connection) {
                instance.filterDbNumber = redis_1.getDatabaseNumberForConnection(connection);
            }
            instance.client = yield redis_1.createRawRedisClient();
            instance.client.on("monitor", instance.monitor.bind(instance));
            yield instance.client.monitorAsync();
            return instance;
        });
    }
}
exports.RedisTestMonitor = RedisTestMonitor;
//# sourceMappingURL=RedisTestMonitor.js.map